@@ -1,7 +1,7 @@
-__version__ = '0.4.6'
+__version__ = '0.5.0'
 
 # amalgamate exclude jupyter_kernel parser_table parser_test_table pyghooks
-# amalgamate exclude winutils wizard pytest_plugin
+# amalgamate exclude winutils wizard pytest_plugin fs
 import os as _os
 if _os.getenv('XONSH_DEBUG', ''):
     pass
@@ -19,8 +19,6 @@ else:
         _sys.modules['xonsh.ansi_colors'] = __amalgam__
         codecache = __amalgam__
         _sys.modules['xonsh.codecache'] = __amalgam__
-        lazyimps = __amalgam__
-        _sys.modules['xonsh.lazyimps'] = __amalgam__
         platform = __amalgam__
         _sys.modules['xonsh.platform'] = __amalgam__
         pretty = __amalgam__
@@ -29,16 +27,18 @@ else:
         _sys.modules['xonsh.timings'] = __amalgam__
         jobs = __amalgam__
         _sys.modules['xonsh.jobs'] = __amalgam__
+        lazyimps = __amalgam__
+        _sys.modules['xonsh.lazyimps'] = __amalgam__
         parser = __amalgam__
         _sys.modules['xonsh.parser'] = __amalgam__
-        teepty = __amalgam__
-        _sys.modules['xonsh.teepty'] = __amalgam__
         tokenize = __amalgam__
         _sys.modules['xonsh.tokenize'] = __amalgam__
         tools = __amalgam__
         _sys.modules['xonsh.tools'] = __amalgam__
         ast = __amalgam__
         _sys.modules['xonsh.ast'] = __amalgam__
+        commands_cache = __amalgam__
+        _sys.modules['xonsh.commands_cache'] = __amalgam__
         contexts = __amalgam__
         _sys.modules['xonsh.contexts'] = __amalgam__
         diff_history = __amalgam__
@@ -57,26 +57,18 @@ else:
         _sys.modules['xonsh.xontribs'] = __amalgam__
         dirstack = __amalgam__
         _sys.modules['xonsh.dirstack'] = __amalgam__
-        history = __amalgam__
-        _sys.modules['xonsh.history'] = __amalgam__
         inspectors = __amalgam__
         _sys.modules['xonsh.inspectors'] = __amalgam__
-        commands_cache = __amalgam__
-        _sys.modules['xonsh.commands_cache'] = __amalgam__
-        environ = __amalgam__
-        _sys.modules['xonsh.environ'] = __amalgam__
-        base_shell = __amalgam__
-        _sys.modules['xonsh.base_shell'] = __amalgam__
-        replay = __amalgam__
-        _sys.modules['xonsh.replay'] = __amalgam__
-        tracer = __amalgam__
-        _sys.modules['xonsh.tracer'] = __amalgam__
         xonfig = __amalgam__
         _sys.modules['xonsh.xonfig'] = __amalgam__
+        environ = __amalgam__
+        _sys.modules['xonsh.environ'] = __amalgam__
+        tracer = __amalgam__
+        _sys.modules['xonsh.tracer'] = __amalgam__
+        replay = __amalgam__
+        _sys.modules['xonsh.replay'] = __amalgam__
         aliases = __amalgam__
         _sys.modules['xonsh.aliases'] = __amalgam__
-        readline_shell = __amalgam__
-        _sys.modules['xonsh.readline_shell'] = __amalgam__
         built_ins = __amalgam__
         _sys.modules['xonsh.built_ins'] = __amalgam__
         execer = __amalgam__
@@ -85,8 +77,12 @@ else:
         _sys.modules['xonsh.imphooks'] = __amalgam__
         shell = __amalgam__
         _sys.modules['xonsh.shell'] = __amalgam__
+        base_shell = __amalgam__
+        _sys.modules['xonsh.base_shell'] = __amalgam__
         main = __amalgam__
         _sys.modules['xonsh.main'] = __amalgam__
+        readline_shell = __amalgam__
+        _sys.modules['xonsh.readline_shell'] = __amalgam__
         del __amalgam__
     except ImportError:
         pass
