@@ -3,6 +3,7 @@
 * completer
 * pip
 * tools
+* xompletions
 * _aliases
 * commands
 * man
@@ -98,13 +99,20 @@ def complete_completer(prefix, line, start, end, ctx):
 #
 # pip
 #
+"""Completers for pip."""
+# pylint: disable=invalid-name, missing-docstring, unsupported-membership-test
+# pylint: disable=unused-argument, not-an-iterable
 re = _LazyModule.load('re', 're')
 subprocess = _LazyModule.load('subprocess', 'subprocess')
 xl = _LazyModule.load('xonsh', 'xonsh.lazyasd', 'xl')
-PIP_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)*"),
-                       globals(), 'PIP_RE')
-PIP_LIST_RE = xl.LazyObject(lambda: re.compile("pip(?:\d|\.)* (?:uninstall|show)"),
-                            globals(), 'PIP_LIST_RE')
+@xl.lazyobject
+def PIP_RE():
+    return re.compile(r"pip(?:\d|\.)*")
+
+
+@xl.lazyobject
+def PIP_LIST_RE():
+    return re.compile(r"pip(?:\d|\.)* (?:uninstall|show)")
 
 
 @xl.lazyobject
@@ -114,7 +122,7 @@ def ALL_COMMANDS():
                                                 stderr=subprocess.DEVNULL))
     except FileNotFoundError:
         return []
-    commands = re.findall("  (\w+)  ", help_text)
+    commands = re.findall(r"  (\w+)  ", help_text)
     return [c for c in commands if c not in ['completion', 'help']]
 
 
@@ -131,7 +139,8 @@ def complete_pip(prefix, line, begidx, endidx, ctx):
         except FileNotFoundError:
             return set()
         items = items.decode('utf-8').splitlines()
-        return set(i.split()[0] for i in items)
+        return set(i.split()[0] for i in items
+                   if i.split()[0].startswith(prefix))
 
     if (line_len > 1 and line.endswith(' ')) or line_len > 2:
         # "pip show " -> no complete (note space)
@@ -177,6 +186,56 @@ def justify(s, max_length, left_pad=0):
     txt = textwrap.wrap(s, width=max_length, subsequent_indent=' '*left_pad)
     return '\n'.join(txt)
 
+#
+# xompletions
+#
+"""Provides completions for xonsh internal utilities"""
+
+xx = _LazyModule.load('xonsh', 'xonsh.xontribs', 'xx')
+xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
+def complete_xonfig(prefix, line, start, end, ctx):
+    """Completion for ``xonfig``"""
+    args = line.split(' ')
+    if len(args) == 0 or args[0] != 'xonfig':
+        return None
+    curix = args.index(prefix)
+    if curix == 1:
+        possible = {'info', 'wizard', 'styles', 'colors', '-h'}
+    elif curix == 2 and args[1] == 'colors':
+        possible = set(xt.color_style_names())
+    else:
+        raise StopIteration
+    return {i for i in possible if i.startswith(prefix)}
+
+
+def _list_installed_xontribs():
+    meta = xx.xontrib_metadata()
+    installed = []
+    for md in meta['xontribs']:
+        name = md['name']
+        spec = xx.find_xontrib(name)
+        if spec is not None:
+            installed.append(spec.name.rsplit('.')[-1])
+
+    return installed
+
+
+def complete_xontrib(prefix, line, start, end, ctx):
+    """Completion for ``xontrib``"""
+    args = line.split(' ')
+    if len(args) == 0 or args[0] != 'xontrib':
+        return None
+    curix = args.index(prefix)
+    if curix == 1:
+        possible = {'list', 'load'}
+    elif curix == 2:
+        if args[1] == 'load':
+            possible = _list_installed_xontribs()
+    else:
+        raise StopIteration
+
+    return {i for i in possible if i.startswith(prefix)}
+
 #
 # _aliases
 #
@@ -357,7 +416,7 @@ POS (optional) is a position into the list of completers at which the new
 #
 os = _LazyModule.load('os', 'os')
 # amalgamated builtins
-xt = _LazyModule.load('xonsh', 'xonsh.tools', 'xt')
+# amalgamated xonsh.tools
 xp = _LazyModule.load('xonsh', 'xonsh.platform', 'xp')
 # amalgamated xonsh.completers.tools
 SKIP_TOKENS = {'sudo', 'time', 'timeit', 'which', 'showcmd', 'man'}
@@ -609,12 +668,16 @@ def _quote_paths(paths, start, end):
     slash = xt.get_sep()
     orig_start = start
     orig_end = end
+    # quote on all or none, to make readline completes to max prefix
+    need_quotes = any(
+        re.search(PATTERN_NEED_QUOTES, x) or
+        (backslash in x and slash != backslash)
+        for x in paths)
+
     for s in paths:
         start = orig_start
         end = orig_end
-        if (start == '' and
-                (re.search(PATTERN_NEED_QUOTES, s) is not None or
-                 (backslash in s and slash != backslash))):
+        if start == '' and need_quotes:
             start = end = _quote_to_use(s)
         if os.path.isdir(expand_path(s)):
             _tail = slash
@@ -812,7 +875,8 @@ def complete_python(prefix, line, start, end, ctx):
     """
     rtn = _complete_python(prefix, line, start, end, ctx)
     if not rtn:
-        prefix = re.split(r'\(|=|{|\[|,', prefix)[-1]
+        prefix = (re.split(r'\(|=|{|\[|,', prefix)[-1] if not
+                  prefix.startswith(',') else prefix)
         start = line.find(prefix)
         rtn = _complete_python(prefix, line, start, end, ctx)
         return rtn, len(prefix)
@@ -969,7 +1033,26 @@ pathlib = _LazyModule.load('pathlib', 'pathlib')
 # amalgamated xonsh.platform
 # amalgamated xonsh.completers.path
 BASH_COMPLETE_SCRIPT = r"""
-{sources}
+{source}
+
+# Override some functions in bash-completion, do not quote for readline
+quote_readline()
+{{
+    echo "$1"
+}}
+
+_quote_readline_by_ref()
+{{
+    if [[ $1 == \'* ]]; then
+        # Leave out first character
+        printf -v $2 %s "${{1:1}}"
+    else
+        printf -v $2 %s "$1"
+    fi
+
+    [[ ${{!2}} == \$* ]] && eval $2=${{!2}}
+}}
+
 
 function _get_complete_statement {{
     complete -p {cmd} 2> /dev/null || echo "-F _minimal"
@@ -999,8 +1082,8 @@ for ((i=0;i<${{#COMPREPLY[*]}};i++)) do echo ${{COMPREPLY[i]}}; done
 
 def complete_from_bash(prefix, line, begidx, endidx, ctx):
     """Completes based on results from BASH completion."""
-    sources = _collect_completions_sources()
-    if not sources:
+    source = _get_completions_source()
+    if not source:
         return set()
 
     if prefix.startswith('$'):  # do not complete env variables
@@ -1024,7 +1107,7 @@ def complete_from_bash(prefix, line, begidx, endidx, ctx):
         prefix_quoted = shlex.quote(prefix)
 
     script = BASH_COMPLETE_SCRIPT.format(
-        sources='\n'.join(sources), line=' '.join(shlex.quote(p) for p in splt),
+        source=source, line=' '.join(shlex.quote(p) for p in splt),
         comp_line=shlex.quote(line), n=n, cmd=shlex.quote(cmd),
         end=endidx + 1, prefix=prefix_quoted, prev=shlex.quote(prev),
     )
@@ -1033,7 +1116,8 @@ def complete_from_bash(prefix, line, begidx, endidx, ctx):
         out = subprocess.check_output(
             [xp.bash_command(), '-c', script], universal_newlines=True,
             stderr=subprocess.PIPE, env=builtins.__xonsh_env__.detype())
-    except (subprocess.CalledProcessError, FileNotFoundError):
+    except (subprocess.CalledProcessError, FileNotFoundError,
+            UnicodeDecodeError):
         return set()
 
     out = out.splitlines()
@@ -1057,17 +1141,12 @@ def complete_from_bash(prefix, line, begidx, endidx, ctx):
     return out, len(prefix) - strip_len
 
 
-def _collect_completions_sources():
-    sources = []
+def _get_completions_source():
     completers = builtins.__xonsh_env__.get('BASH_COMPLETIONS', ())
-    paths = (pathlib.Path(x) for x in completers)
-    for path in paths:
+    for path in map(pathlib.Path, completers):
         if path.is_file():
-            sources.append('source "{}"'.format(path.as_posix()))
-        elif path.is_dir():
-            for _file in (x for x in path.glob('*') if x.is_file()):
-                sources.append('source "{}"'.format(_file.as_posix()))
-    return sources
+            return 'source "{}"'.format(path.as_posix())
+    return None
 
 #
 # dirs
@@ -1108,6 +1187,7 @@ def complete_rmdir(prefix, line, start, end, ctx):
 # amalgamated xonsh.completers.python
 # amalgamated xonsh.completers.commands
 # amalgamated xonsh.completers.completer
+# amalgamated xonsh.completers.xompletions
 def default_completers():
     """Creates a copy of the default completers."""
     return collections.OrderedDict([
@@ -1118,6 +1198,8 @@ def default_completers():
         ('pip', complete_pip),
         ('cd', complete_cd),
         ('rmdir', complete_rmdir),
+        ('xonfig', complete_xonfig),
+        ('xontrib', complete_xontrib),
         ('bash', complete_from_bash),
         ('man', complete_from_man),
         ('import', complete_import),
