@@ -32,6 +32,22 @@ def XONSH_TOKENS():
 
 
 def complete_python(prefix, line, start, end, ctx):
+    """
+    Completes based on the contents of the current Python environment,
+    the Python built-ins, and xonsh operators.
+    If there are no matches, split on common delimiters and try again.
+    """
+    rtn = _complete_python(prefix, line, start, end, ctx)
+    if not rtn:
+        prefix = (re.split(r'\(|=|{|\[|,', prefix)[-1] if not
+                  prefix.startswith(',') else prefix)
+        start = line.find(prefix)
+        rtn = _complete_python(prefix, line, start, end, ctx)
+        return rtn, len(prefix)
+    return rtn
+
+
+def _complete_python(prefix, line, start, end, ctx):
     """
     Completes based on the contents of the current Python environment,
     the Python built-ins, and xonsh operators.
@@ -58,6 +74,8 @@ def complete_python_mode(prefix, line, start, end, ctx):
         return set()
     prefix_start = prefix[:2]
     python_matches = complete_python(prefix[2:], line, start+2, end, ctx)
+    if isinstance(python_matches, cabc.Sequence):
+        python_matches = python_matches[0]
     return set(prefix_start + i for i in python_matches)
 
 
